@@ -8,4 +8,10 @@
 #'   values variables. Further identifying and values variables would be added
 #'   when more variables are contained in a table.
 #'
-"schema_default"
\ No newline at end of file
+"schema_default"
+
+#' List of table types
+#'
+#' @format The object of class \code{list} contains 20 different types of tables
+#'   that are used throughout the unit-tests and examples/vignette.
+"tabs2shift"
\ No newline at end of file
