@@ -1,20 +1,98 @@
 #' The \code{schema} class (S4) and its methods
 #'
-#' A \code{schema} stores the information of where which information is stored in
-#' a table of data.
-#' @slot cluster [\code{list(1)}]\cr description of clusters of data.
-#' @slot header [code{list(1)}]\cr description of the header.
-#' @slot meta [\code{list(1)}]\cr description of the metadata.
-#' @slot variables [\code{named list(.)}]\cr description of variables.
-#' @details The slot \code{variables} typically contains several lists that each
-#' record the metadata of the respective variables.
+#' A \code{schema} stores the information of where which information is stored
+#' in a table of data.
+#' @slot cluster [\code{list(1)}]\cr description of
+#'   \code{\link[=setCluster]{clusters}} in the table.
+#' @slot format [\code{list(1)}]\cr description of the table
+#'   \code{\link[=setFormat]{format}}
+#' @slot variables [\code{named list(.)}]\cr description of
+#'   \code{\link[=setIDVar]{identifying}} and \code{\link[=setObsVar]{observed}}
+#'   variables.
+#' @section Setting up schema descriptions: This section outlines the currently
+#'   recommended strategy for setting up schema descriptions. For example tables
+#'   and the respective schemas, see the vignette.
+#'
+#'   \enumerate{ \item \emph{Variables}: Clarify which are the identifying
+#'   variables and which are the observed variables. Make sure not to mistake a
+#'   listed observed variable as identifying variable.
+#'
+#'   \item \emph{Clusters}: Determine whether there are clusters and if so, find
+#'   the origin (top left cell) of each cluster and provide the required
+#'   information in \code{\link[=setCluster]{setCluster}(top = ..., left =
+#'   ...)}. It is advised to treat a table that contains meta-data in the top
+#'   rows as cluster, as this is often the case with implicit variables. All
+#'   variables need to be specified in each cluster (in case clusters are all
+#'   organised in the same arrangement), or \code{relative = TRUE} can be used.
+#'   Data may be organised into clusters a) whenever a set of variables occurs
+#'   more than once in the same table, nested into another variable, or b) when
+#'   the data are organised into separate spreadsheets or files according to one
+#'   of the variables (depending on the context, these issues can also be solved
+#'   differently). In both cases the variable responsible for clustering (the
+#'   cluster ID) can be either an identifying variable, or a categorical
+#'   observed variable: \itemize{
+#'
+#'   \item in case the cluster ID is an identifying variable, provide its name
+#'   in \code{\link[=setCluster]{setCluster(id = ...)}} and specify it as an
+#'   identifying variable (\code{\link{setIDVar}})
+#'
+#'   \item in case it is a observed variable, provide simply
+#'   \code{\link[=setCluster]{setCluster}(..., id = "observed")}. }
+#'
+#'   \item \emph{Meta-data}: Provide potentially information about the format
+#'   (\code{\link{setFormat}}).
+#'
+#'   \item \emph{Identifying variables}: Determine the following: \itemize{
+#'
+#'   \item is the variable available at all? This is particularly important when
+#'   the data are split up into tables that are in spreadsheets or files. Often
+#'   the variable that splits up the data (and thus identifies the clusters) is
+#'   not explicitly available in the table anymore. In such a case, provide the
+#'   value in \code{\link[=setIDVar]{setIDVar}(..., value = ...)}.
+#'
+#'   \item all columns in which the variable values sit.
+#'
+#'   \item in case the variable is in several columns, determine additionally
+#'   the row in which its values sit. In this case, the values will look like
+#'   they are part of a header.
+#'
+#'   \item in case the variable must be split off of another column, provide a
+#'   regular expression that results in the target subset via
+#'   \code{\link[=setIDVar]{setIDVar}(..., split = ...)}.
+#'
+#'   \item in case the variable is distinct from the main table, provide the
+#'   explicit (non-relative) position and set
+#'   \code{\link[=setIDVar]{setIDVar}(..., distinct = TRUE)}. }
+#'
+#'   \item \emph{Observed variable}: Determine the following: \itemize{
+#'
+#'   \item all columns in which the values of the variable sit.
+#'
+#'   \item the unit and conversion factor.
+#'
+#'   \item in case the variable is not tidy, go through the following cases one
+#'   after the other: \itemize{
+#'
+#'   \item in case the variable is nested in a wide identifying variable,
+#'   determine in addition to the columns in which the values sit also the rows
+#'   in which the \emph{variable name} sits.
+#'
+#'   \item in case the names of the variable are given as a value of an
+#'   identifying variable, give the column name as
+#'   \code{\link[=setObsVar]{setObsVar}(..., key = ...)}, together with the name
+#'   of the respective observed variable (as it appears in the table) in
+#'   \code{values}.
+#'
+#'   \item in case the name of the variable is the ID of clusters, specify
+#'   \code{\link[=setObsVar]{setObsVar}(..., key = "cluster", value = ...)},
+#'   where \code{values} has the cluster number the variable refers to. } } }
 #' @importFrom rlang is_integerish
 #' @importFrom stringr str_sub
 
 schema <- setClass(Class = "schema",
                    slots = c(clusters = "list",
-                             header = "list",
-                             meta = "list",
+                             format = "list",
+                             filter = "list",
                              variables = "list"
                    )
 )
@@ -29,8 +107,8 @@ setValidity(Class = "schema", function(object){
     if(!is.list(object@clusters)){
       errors <- c(errors, "the slot 'clusters' is not a list.")
     }
-    if(!all(names(object@clusters) %in% c("row", "col", "width", "height", "id"))){
-      errors <- c(errors, "'names(schema$clusters)' must be a permutation of set {row,col,width,height,id}")
+    if(!all(names(object@clusters) %in% c("id", "group", "row", "col", "width", "height", "member"))){
+      errors <- c(errors, "'names(schema$clusters)' must be a permutation of set {id,group,row,col,width,height,member}")
     }
     if(!is.null(object@clusters$row)){
       if(!is.numeric(object@clusters$row)){
@@ -57,67 +135,76 @@ setValidity(Class = "schema", function(object){
         errors <- c(errors, "'schema$clusters$id' must have a character value.")
       }
     }
-  }
-
-  if(!.hasSlot(object = object, name = "header")){
-    errors <- c(errors, "the schema does not have a 'header' slot.")
-  } else {
-    if(!is.list(object@header)){
-      errors <- c(errors, "the slot 'header' is not a list.")
-    }
-    if(length(object@header) == 0){
-      errors <- c(errors, "the slot 'header' does not contain any entries.")
-    }
-    if(!all(names(object@header) %in% c("row", "rel", "merge"))){
-      errors <- c(errors, "'names(header)' must be a permutation of set {row,rel,merge}")
-    }
-    if(!is.null(object@header$row)){
-      if(!is.numeric(object@header$row)){
-        errors <- c(errors, "'header$row' must have a numeric value.")
+    if(!is.null(object@clusters$group)){
+      if(!is.character(object@clusters$group)){
+        errors <- c(errors, "'schema$clusters$group' must have a character value.")
       }
     }
-    if(!is.logical(object@header$rel)){
-      errors <- c(errors, "'header$rel' must have a logical value.")
-    }
-    if(!is.logical(object@header$merge)){
-      errors <- c(errors, "'header$merge' must have a logical value.")
+    if(!is.null(object@clusters$member)){
+      if(!is.numeric(object@clusters$member)){
+        errors <- c(errors, "'schema$clusters$member' must have a numeric value.")
+      }
     }
   }
 
-  if(!.hasSlot(object = object, name = "meta")){
-    errors <- c(errors, "the schema does not have a 'meta' slot.")
+  if(!.hasSlot(object = object, name = "format")){
+    errors <- c(errors, "the schema does not have a 'format' slot.")
   } else {
-    if(!is.list(object@meta)){
-      errors <- c(errors, "the slot 'meta' is not a list.")
+    if(!is.list(object@format)){
+      errors <- c(errors, "the slot 'format' is not a list.")
     }
-    if(length(object@meta) == 0){
-      errors <- c(errors, "the slot 'meta' does not contain any entries.")
+    if(length(object@format) == 0){
+      errors <- c(errors, "the slot 'format' does not contain any entries.")
     }
-    if(!all(names(object@meta) %in% c("del", "dec", "na"))){
-      errors <- c(errors, "'names(schema$meta)' must be a permutation of set {del,dec,na}")
+    if(!all(names(object@format) %in% c("del", "dec", "na"))){
+      errors <- c(errors, "'names(schema$format)' must be a permutation of set {del,dec,na}")
     }
-    if(!is.null(object@meta$del)){
-      if(!is.character(object@meta$del)){
-        errors <- c(errors, "'schema$meta$del' must have a character value.")
+    if(!is.null(object@format$del)){
+      if(!is.character(object@format$del)){
+        errors <- c(errors, "'schema$format$del' must have a character value.")
       }
     }
-    if(!is.null(object@meta$dec)){
-      if(!is.character(object@meta$dec)){
-        errors <- c(errors, "'schema$meta$dec' must have a character value.")
+    if(!is.null(object@format$dec)){
+      if(!is.character(object@format$dec)){
+        errors <- c(errors, "'schema$format$dec' must have a character value.")
       }
     }
-    if(!is.null(object@meta$na)){
-      if(!is.character(object@meta$na)){
-        errors <- c(errors, "'schema$meta$na' must have a character value.")
+    if(!is.null(object@format$na)){
+      if(!is.character(object@format$na)){
+        errors <- c(errors, "'schema$format$na' must have a character value.")
       }
     }
-    # if(!is.null(object@meta$types)){
-    #   if(!is.character(object@meta$types)){
-    #     errors <- c(errors, "'schema$meta$types' must have a character value.")
+    # if(!is.null(object@format$types)){
+    #   if(!is.character(object@format$types)){
+    #     errors <- c(errors, "'schema$format$types' must have a character value.")
     #   }
     # }
   }
 
+  if(!.hasSlot(object = object, name = "filter")){
+    errors <- c(errors, "the schema does not have a 'filter' slot.")
+  } else {
+    if(!is.list(object@filter)){
+      errors <- c(errors, "the slot 'filter' is not a list.")
+    }
+    if(length(object@filter) == 0){
+      errors <- c(errors, "the slot 'filter' does not contain any entries.")
+    }
+    if(!all(names(object@filter) %in% c("row", "col", "invert"))){
+      errors <- c(errors, "'names(schema$filter)' must be a permutation of set {row,col,invert}")
+    }
+    if(!is.null(object@filter$row)){
+      if(!is.numeric(object@filter$row)){
+        errors <- c(errors, "'schema$filter$row' must have a numeric value.")
+      }
+    }
+    if(!is.null(object@filter$col)){
+      if(!is.numeric(object@filter$col)){
+        errors <- c(errors, "'schema$filter$col' must have a numeric value.")
+      }
+    }
+  }
+
   if(!.hasSlot(object = object, name = "variables")){
     errors <- c(errors, "the schema does not have a 'variables' slot.")
   } else {
@@ -131,14 +218,14 @@ setValidity(Class = "schema", function(object){
       theVariable <- object@variables[[i]]
       theName <- names(object@variables)[i]
 
-      if(!theVariable$type %in% c("id", "measured")){
-        errors <- c(errors, paste0("the variables '", theName, "' does must be of type 'id' or 'measured'."))
+      if(!theVariable$type %in% c("id", "observed")){
+        errors <- c(errors, paste0("the variables '", theName, "' does must be of type 'id' or 'observed'."))
         return(paste0("\n", errors))
       }
 
       if(theVariable$type == "id"){
-        if(!all(names(theVariable) %in% c("type", "value", "split", "row", "col", "rel", "dist"))){
-          errors <- c(errors, paste0("'names(", theName, ")' must be a permutation of set {type,value,split,row,col,rel,dist}"))
+        if(!all(names(theVariable) %in% c("type", "value", "row", "col", "rel", "split", "dist", "merge"))){
+          errors <- c(errors, paste0("'names(", theName, ")' must be a permutation of set {type,value,row,col,rel,split,merge,dist}"))
         }
         if(!is.null(theVariable$value)){
           if(!is.character(theVariable$value)){
@@ -151,12 +238,12 @@ setValidity(Class = "schema", function(object){
           }
         }
         if(!is.null(theVariable$row)){
-          if(!is.numeric(theVariable$row)){
+          if(!(is.numeric(theVariable$row) | testClass(x = theVariable$row, classes = "quosure"))){
             errors <- c(errors, paste0("'", theName, "$row' must have a numeric value."))
           }
         }
         if(!is.null(theVariable$col)){
-          if(!is.numeric(theVariable$col)){
+          if(!(is.numeric(theVariable$col) | testClass(x = theVariable$col, classes = "quosure"))){
             errors <- c(errors, paste0("'", theName, "$col' must have a numeric value."))
           }
         }
@@ -171,23 +258,23 @@ setValidity(Class = "schema", function(object){
         if(!all(names(theVariable) %in% c("type", "unit", "factor", "row", "col", "rel", "dist", "key", "value"))){
           errors <- c(errors, paste0("'names(", theName, ")' must be a permutation of set {type,unit,factor,row,col,rel,dist,key,value}"))
         }
-        if(!is.null(theVariable$unit)){
-          if(!is.character(theVariable$unit)){
-            errors <- c(errors, paste0("'", theName, "$unit' must have a character value."))
-          }
-        }
+        # if(!is.null(theVariable$unit)){
+        #   if(!is.character(theVariable$unit)){
+        #     errors <- c(errors, paste0("'", theName, "$unit' must have a character value."))
+        #   }
+        # }
         if(!is.null(theVariable$factor)){
           if(!is.numeric(theVariable$factor)){
             errors <- c(errors, paste0("'", theName, "$factor' must have a numeric value."))
           }
         }
         if(!is.null(theVariable$row)){
-          if(!is.numeric(theVariable$row)){
+          if(!(is.numeric(theVariable$row) | testClass(x = theVariable$row, classes = "quosure"))){
             errors <- c(errors, paste0("'", theName, "$row' must have a numeric value."))
           }
         }
         if(!is.null(theVariable$col)){
-          if(!is.numeric(theVariable$col)){
+          if(!(is.numeric(theVariable$col) | testClass(x = theVariable$col, classes = "quosure"))){
             errors <- c(errors, paste0("'", theName, "$col' must have a numeric value."))
           }
         }
@@ -197,11 +284,11 @@ setValidity(Class = "schema", function(object){
         if(!is.logical(theVariable$dist)){
           errors <- c(errors, paste0("'", theName, "$dist' must either be 'TRUE' or 'FALSE'."))
         }
-        if(!is.null(theVariable$key)){
-          if(!is.character(theVariable$key)){
-            errors <- c(errors, paste0("'", theName, "$key' must have a character value."))
-          }
-        }
+        # if(!is.null(theVariable$key)){
+        #   if(!is.character(theVariable$key)){
+        #     errors <- c(errors, paste0("'", theName, "$key' must have a character value."))
+        #   }
+        # }
         if(!is.null(theVariable$value)){
           if(theVariable$key == "cluster"){
             if(!rlang::is_integerish(theVariable$value)){
@@ -231,6 +318,7 @@ setValidity(Class = "schema", function(object){
 #' @param object [\code{schema}]\cr the schema to print.
 #' @importFrom crayon yellow
 #' @importFrom stringr str_split
+#' @importFrom rlang eval_tidy is_quosure prim_name
 
 setMethod(f = "show",
           signature = "schema",
@@ -257,8 +345,9 @@ setMethod(f = "show",
               } else {
                 top <- clusters$row
               }
-              clusterSpecs <- paste0("\n    origin: ", paste(top, left, collapse = ", ", sep = "|"), "  (row|col)",
-                                     ifelse(!is.null(clusters$id), paste0("\n    id    : ", clusters$id), ""))
+              clusterSpecs <- paste0("\n    origin : ", paste(top, left, collapse = ", ", sep = "|"), "  (row|col)",
+                                     ifelse(!is.null(clusters$group), paste0("\n    groups : ", clusters$group), ""),
+                                     ifelse(!is.null(clusters$id), paste0("\n    id     : ", clusters$id), ""))
             }
             cat(paste0("  ", nClusters, " ", nClustName, clusterSpecs, "\n\n"))
 
@@ -279,6 +368,8 @@ setMethod(f = "show",
             theRows <- sapply(seq_along(variables), function(x){
               if(is.null(variables[[x]]$row)){
                 ""
+              } else if(is_quosure(variables[[x]]$row)){
+                prim_name(eval_tidy(variables[[x]]$row))
               } else {
                 temp <- unique(variables[[x]]$row)
                 # make a short sequence of 'theRows'
@@ -304,10 +395,16 @@ setMethod(f = "show",
             theCols <- sapply(seq_along(variables), function(x){
               if(is.null(variables[[x]]$col)){
                 ""
+              } else if(is_quosure(variables[[x]]$col)){
+                prim_name(eval_tidy(variables[[x]]$col))
               } else {
                 temp <- unique(variables[[x]]$col)
                 # make a short sequence of 'theRows'
-                dists <- temp - c(temp[1]-1, temp)[-(length(temp)+1)]
+                if(is.numeric(temp)){
+                  dists <- temp - c(temp[1]-1, temp)[-(length(temp)+1)]
+                } else {
+                  dists <- 0
+                }
                 if(all(dists == 1) & length(temp) > 1){
                   paste0(min(temp), ":", max(temp))
                 } else {
@@ -316,7 +413,7 @@ setMethod(f = "show",
               }
             })
             nCols <- sapply(seq_along(theCols), function(x){
-              ifelse(test = is.null(theCols[[x]]) , yes = 0, no = nchar(paste0(theCols[[x]], collapse = ", ")))
+              ifelse(test = is.null(theCols[[x]]), yes = 0, no = nchar(paste0(theCols[[x]], collapse = ", ")))
             })
             maxCols <- ifelse(any(nCols > 3), max(nCols), 3)
             if(any(nCols != 0)){
@@ -382,13 +479,21 @@ setMethod(f = "show",
             theRels <- sapply(seq_along(variables), function(x){
               str_sub(as.character(variables[[x]]$rel), 1, 1)
             })
-            included <- c(included, TRUE)
+            if(all(theRels == "F")){
+              included <- c(included, FALSE)
+            } else {
+              included <- c(included, TRUE)
+            }
 
             # whether variables are distinct
             theDist <- sapply(seq_along(variables), function(x){
               str_sub(as.character(variables[[x]]$dist), 1, 1)
             })
-            included <- c(included, TRUE)
+            if(all(theDist == "F")){
+              included <- c(included, FALSE)
+            } else {
+              included <- c(included, TRUE)
+            }
 
             for(i in 1:(length(variables)+1)){
 
@@ -422,13 +527,21 @@ setMethod(f = "show",
                 } else {
                   head6 <- line6 <- ""
                 }
-                head7 <- paste0("rel   ")
-                head8 <- paste0("dist")
+                if(included[7]){
+                  head7 <- paste0("rel   ")
+                  line7 <- paste0(c(rep("-", 5), " "), collapse = "")
+                } else {
+                  head7 <- line7 <- ""
+                }
+                if(included[8]){
+                  head8 <-paste0("dist")
+                  line8 <- paste0(c(rep("-", 6), " "), collapse = "")
+                } else {
+                  head8 <- line8 <- ""
+                }
 
                 cat(paste0(head1, head2, head3, head4, head5, head6, head7, head8), "\n")
-                cat(" ", paste0(line1, line2, line3, line4, line5, line6,
-                               paste0(c(rep("-", 5), " "), collapse = ""),
-                               paste0(c(rep("-", 6), " "), collapse = "")), "\n")
+                cat(" ", paste0(line1, line2, line3, line4, line5, line6, line7, line8), "\n")
 
               } else {
 
@@ -459,8 +572,16 @@ setMethod(f = "show",
                 } else {
                   var6 <- ""
                 }
-                var7 <- paste0(theRels[[i-1]], "     ")
-                var8 <- paste0(theDist[[i-1]], "  ")
+                if(included[7]){
+                  var7 <- paste0(theRels[[i-1]], "     ")
+                } else {
+                  var7 <- ""
+                }
+                if(included[8]){
+                  var8 <- paste0(theDist[[i-1]], "  ")
+                } else {
+                  var8 <- ""
+                }
 
                 cat(paste0(var1, var2, var3, var4, var5, var6, var7, var8, "\n"))
 
